﻿using System;
using System.IO;
using System.Xml.Serialization;
using System.Text;
using System.Collections.Generic;

namespace DataPorter_WebService_Consumer
{
  class Program
  {
    static OPCAdvancedDataAccess.OPCAdvancedDataAccess opcDA;
    static void GetValueOneItem( string ItemName, bool UseXmlOutput )
    {
      OPCAdvancedDataAccess.ItemValue val = opcDA.GetValue_OneItem( ItemName );
      DisplayOneItemValue( ItemName, val, UseXmlOutput );
    }
    static void GetValuesManyItems( string[] ItemNames, bool UseXmlOutput )
    {
      OPCAdvancedDataAccess.ItemValue[] values = opcDA.GetValue_MultiItem( ItemNames );
      foreach ( var val in values )
        DisplayOneItemValue( val.ItemName, val, UseXmlOutput );
    }
    private static void DisplayOneItemValue( string ItemName, OPCAdvancedDataAccess.ItemValue val, bool UseXmlOutput )
    {
      if ( UseXmlOutput )
      {
        StringBuilder sb = new StringBuilder();
        StringWriter sw = new StringWriter( sb );
        XmlSerializer serializer = new XmlSerializer( typeof( OPCAdvancedDataAccess.ItemValue ) );
        serializer.Serialize( sw, val );
        Console.WriteLine( sb.ToString() );
      }
      else
      {
        Console.WriteLine( "Item {0}; {1}; {2}; {3}",
          ItemName, val.Value, val.Quality.QualityBits, val.Timestamp );
      }
    }
    private static string[] GetAvailiableItems( bool Display, bool UseXmlOutput )
    {
      string[] items = opcDA.GetAvailiableItems();
      if ( Display )
      {
        //displaing the list
        if ( UseXmlOutput )
        {
          StringBuilder sb = new StringBuilder();
          StringWriter sw = new StringWriter( sb );
          XmlSerializer serializer = new XmlSerializer( typeof( string[] ) );
          serializer.Serialize( sw, items );
          Console.WriteLine( sb.ToString() );
        }
        else
        {
          foreach ( string item in items )
            Console.WriteLine( item );
        }
      }
      return items;
    }

    static void Main( string[] args )
    {
      string commandline = System.Environment.CommandLine;
      if ( commandline.Contains( "--help" ) || args==null || args.Length==0)
      {
        Console.WriteLine( "Usage:" );
        Console.WriteLine( "OPCClient --help" );
        Console.WriteLine( "OPCClient --list [--xml]" );
        Console.WriteLine( "OPCClient item_names [--xml]" );
        Console.WriteLine( "--help - dispalys this help information" );
        Console.WriteLine( "--xml - enables output as xml" );
        Console.WriteLine( "--list - displays list of items" );
        Console.WriteLine( "item_names - list of item names, that we want to read value separated by spaces " );
        return;
      }
      bool UseXmlOutput = false;
      if ( commandline.Contains( "--xml" ) )
      {
        UseXmlOutput = true;
      }
      //connecting to remote object
      opcDA = new OPCAdvancedDataAccess.OPCAdvancedDataAccess();
      if ( commandline.Contains( "--list" ) )
      {
        GetAvailiableItems( true, UseXmlOutput );
        return;
      }
      //now we can get items values
      //if we are not using the XML output option we can pass arguments array 
      //if we are using XML output, we have to remove --xml from the arguments array;
      if ( UseXmlOutput )
      {
        List<string> list = new List<string>( args );
        list.Remove( "--xml" );
        GetValuesManyItems( list.ToArray(), UseXmlOutput );
      }
      else
      {
        GetValuesManyItems( args, UseXmlOutput );
      }
    }

  }
}
